@@ -13,8 +13,8 @@
         >>> oT.routine("9n", "C:\\Users\\UserName\\Documents\\GitHub\\openTEPES", "glpk")
 """
 
-__version__ = '2.0.24'
-from .openTEPES_run import main
+__version__ = '2.1.0'
+from .openTEPES_Main import main
 from .openTEPES import *
 from .openTEPES_InputData import *
 from .openTEPES_ModelFormulation import *
