@@ -65,7 +65,7 @@
 # make it effectively proprietary.  To prevent this, the GPL assures that
 # patents cannot be used to render the program non-free.
 
-# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 2.0.6 - February 23, 2021
+# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 2.1.0 - March 13, 2021
 # simplicity and transparency in power systems planning
 
 # Developed by
@@ -91,21 +91,21 @@ import setuptools
 
 from   pyomo.environ import ConcreteModel, Set
 
-from openTEPES.openTEPES_InputData import InputData
-from openTEPES.openTEPES_ModelFormulation import InvestmentModelFormulation, OperationModelFormulation
-from openTEPES.openTEPES_OutputResults import OutputResults
-from openTEPES.openTEPES_ProblemSolving import ProblemSolving
+from openTEPES.openTEPES_InputData        import InputData
+from openTEPES.openTEPES_ModelFormulation import InvestmentModelFormulation, GenerationOperationModelFormulation, NetworkOperationModelFormulation
+from openTEPES.openTEPES_ProblemSolving   import ProblemSolving
+from openTEPES.openTEPES_OutputResults    import *
 
 
-def execution(CaseName, DirName, SolverName):
+def openTEPES_run(DirName, CaseName, SolverName):
 
     InitialTime = time.time()
     _path = os.path.join(DirName, CaseName)
 
     #%% model declaration
-    mTEPES = ConcreteModel('Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 2.0.6 - February 23, 2021')
+    mTEPES = ConcreteModel('Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 2.1.0 - March 13, 2021')
 
-    InputData(CaseName, DirName, mTEPES)
+    InputData(DirName, CaseName, mTEPES)
 
     # investment model objective function
     InvestmentModelFormulation(mTEPES)
@@ -125,7 +125,8 @@ def execution(CaseName, DirName, SolverName):
         mTEPES.n2 = Set(initialize=mTEPES.nn , ordered=True, doc='load levels', filter=lambda mTEPES,nn : nn  in list(mTEPES.pDuration) and mTEPES.nn.ord(nn) > (st-1)*mTEPES.pStageDuration and mTEPES.nn.ord(nn) <= st*mTEPES.pStageDuration)
 
         # operation model objective function and constraints by stage
-        OperationModelFormulation(mTEPES, st)
+        GenerationOperationModelFormulation(mTEPES, st)
+        NetworkOperationModelFormulation   (mTEPES, st)
 
     StartTime = time.time()
     mTEPES.write(_path+'/openTEPES_'+CaseName+'.lp', io_options={'symbolic_solver_labels': True})  # create lp-format file
@@ -133,7 +134,7 @@ def execution(CaseName, DirName, SolverName):
     StartTime         = time.time()
     print('Writing LP file                       ... ', round(WritingLPFileTime), 's')
 
-    ProblemSolving(CaseName, DirName, SolverName, mTEPES)
+    ProblemSolving(DirName, CaseName, SolverName, mTEPES)
 
     mTEPES.del_component(mTEPES.sc)
     mTEPES.del_component(mTEPES.p )
@@ -142,7 +143,14 @@ def execution(CaseName, DirName, SolverName):
     mTEPES.p  = Set(initialize=mTEPES.pp , ordered=True, doc='periods'                                                                                 )
     mTEPES.n  = Set(initialize=mTEPES.nn , ordered=True, doc='load levels', filter=lambda mTEPES,nn : nn  in list(mTEPES.pDuration)                    )
 
-    OutputResults(CaseName, DirName, mTEPES)
+    InvestmentResults(DirName, CaseName, mTEPES)
+    GenerationOperationResults(DirName, CaseName, mTEPES)
+    ESSOperationResults(DirName, CaseName, mTEPES)
+    FlexibilityResults(DirName, CaseName, mTEPES)
+    NetworkOperationResults(DirName, CaseName, mTEPES)
+    MarginalResults(DirName, CaseName, mTEPES)
+    EconomicResults(DirName, CaseName, mTEPES)
+    NetworkMapResults(DirName, CaseName, mTEPES)
 
     TotalTime = time.time() - InitialTime
     print('Total time                            ... ', round(TotalTime), 's')
