@@ -1,9 +1,10 @@
-# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - February 21, 2021
+# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - March 18, 2021
 
 import time
 from   collections   import defaultdict
 from   pyomo.environ import Set, Constraint, Objective, Block, minimize
 
+
 def InvestmentModelFormulation(mTEPES):
     print('Investment model formulation ****')
 
@@ -21,8 +22,9 @@ def InvestmentModelFormulation(mTEPES):
     StartTime        = time.time()
     print('Generating investment o.f.            ... ', round(GeneratingOFTime), 's')
 
-def OperationModelFormulation(mTEPES, st):
-    print('Operation  model formulation ****')
+
+def GenerationOperationModelFormulation(mTEPES, st):
+    print('Genr oper  model formulation ****')
 
     StartTime = time.time()
 
@@ -322,6 +324,12 @@ def OperationModelFormulation(mTEPES, st):
     StartTime           = time.time()
     print('Generating minimum up/down time       ... ', round(GeneratingMinUDTime), 's')
 
+
+def NetworkOperationModelFormulation(mTEPES, st):
+    print('Netw oper  model formulation ****')
+
+    StartTime = time.time()
+
     #%%
     def eInstalNetCap1(mTEPES,sc,p,n,ni,nf,cc):
         return mTEPES.vFlow[sc,p,n,ni,nf,cc] / max(mTEPES.pLineNTCBck[ni,nf,cc],mTEPES.pLineNTCFrw[ni,nf,cc]) >= - mTEPES.vNetworkInvest[ni,nf,cc]
